CREATE TABLE `record_types` (
  `id` int(64) unsigned NOT NULL auto_increment,
  `name` varchar(40) NOT NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `names` (
`id` int(64) unsigned NOT NULL auto_increment,
`name` varchar(255) NOT NULL,
`normalized_name` varchar(255) NOT NULL ,
PRIMARY KEY(`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `authority_records` (
  `id` int(64) unsigned NOT NULL auto_increment,
  `generated` int(1) NOT NULL,
  `xml_hashcode` int(32) default NULL,
  `orig_xml` longtext,
  `record_type_id` int(64) unsigned default NULL,
  `authority_name_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `record_type_id` (`record_type_id`),
  KEY `authority_name_id` (`authority_name_id`),
  CONSTRAINT `authority_records_ibfk_1` FOREIGN KEY (`record_type_id`) REFERENCES `record_types` (`id`),
  CONSTRAINT `authority_records_ibfk_3` FOREIGN KEY (`authority_name_id`) REFERENCES `names` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `authority_records_alter_forms` (
  `name` int(64) unsigned NOT NULL,
  `record_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`name`, `record_id`),
  KEY `record_id` (`record_id`),
  KEY `name` (`name`),
  CONSTRAINT `authority_records_alter_forms_ibfk_1` FOREIGN KEY (`record_id`) REFERENCES `authority_records` (`id`)  ON DELETE CASCADE,
  CONSTRAINT `authority_records_alter_forms_ibfk_2` FOREIGN KEY (`name`) REFERENCES `names` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `authority_records_see_also` (
  `name` int(64) unsigned NOT NULL,
  `record_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`name`, `record_id`),
  KEY `record_id` (`record_id`),
  KEY `name` (`name`),
  CONSTRAINT `authority_records_see_also_ibfk_1` FOREIGN KEY (`record_id`) REFERENCES `authority_records` (`id`)  ON DELETE CASCADE,
  CONSTRAINT `authority_records_see_also_ibfk_2` FOREIGN KEY (`name`) REFERENCES `names` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records` (
  `id` int(64) unsigned NOT NULL auto_increment,
  `processed` int(1) NOT NULL,
  `xml_hashcode` int(32) default NULL,
  `orig_xml` longtext,
  `record_type_id` int(10) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `record_type_id` (`record_type_id`),
  CONSTRAINT `bib_records_ibfk_1` FOREIGN KEY (`record_type_id`) REFERENCES `record_types` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records_titles` (
  `name` int(64) unsigned NOT NULL,
  `record_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`name`, `record_id`),
  KEY `record_id` (`record_id`),
  KEY `name` (`name`),
  CONSTRAINT `bib_records_titles_ibfk_1` FOREIGN KEY (`record_id`) REFERENCES `bib_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `bib_records_titles_ibfk_2` FOREIGN KEY (`name`) REFERENCES `names` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records_authors` (
  `name` int(64) unsigned NOT NULL,
  `record_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`name`, `record_id`),
  KEY `record_id` (`record_id`),
  KEY `name` (`name`),
  CONSTRAINT `bib_name_ibfk_1` FOREIGN KEY (`record_id`) REFERENCES `bib_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `bib_name_ibfk_2` FOREIGN KEY (`name`) REFERENCES `names` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records_subjects` (
  `name` int(64) unsigned NOT NULL,
  `record_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`name`, `record_id`),
  KEY `record_id` (`record_id`),
  KEY `name` (`name`),
  CONSTRAINT `bib_records_subjects_ibfk_1` FOREIGN KEY (`record_id`) REFERENCES `bib_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `bib_records_subjects_ibfk_2` FOREIGN KEY (`name`) REFERENCES `names` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `authority_records_links` (
  `id` int(64) unsigned NOT NULL auto_increment,
  `flagged` int(1) unsigned default NULL,
  `status` int(1) unsigned default NULL,
  `auth_record_id` int(64) unsigned NOT NULL,
  `evidence` text,
  `time_found` datetime default NULL,
  `time_verified` datetime default NULL,
  `approvedBy` varchar(40) default NULL,
  `percent_confidence` int(3) default NULL,
  `string_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`id`),
  KEY `auth_record_id` (`auth_record_id`),
  KEY `string_id` (`string_id`),
  CONSTRAINT `authority_records_links_ibfk_2` FOREIGN KEY (`auth_record_id`) REFERENCES `authority_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `authority_records_links_ibfk_3` FOREIGN KEY (`string_id`) REFERENCES `names` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records_authors_links` (
  `bib_record_id` int(64) unsigned NOT NULL,
  `auth_link_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`bib_record_id`, `auth_link_id`),
  KEY `bib_record_id` (`bib_record_id`),
  KEY `auth_link_id` (`auth_link_id`),
  CONSTRAINT `bib_records_author_links_ibfk_1` FOREIGN KEY (`bib_record_id`) REFERENCES `bib_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `bib_records_author_links_ibfk_2` FOREIGN KEY (`auth_link_id`) REFERENCES `authority_records_links` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;

CREATE TABLE `bib_records_subjects_links` (
  `bib_record_id` int(64) unsigned NOT NULL,
  `auth_link_id` int(64) unsigned NOT NULL,
  PRIMARY KEY  (`bib_record_id`, `auth_link_id`),
  KEY `bib_record_id` (`bib_record_id`),
  KEY `auth_link_id` (`auth_link_id`),
  CONSTRAINT `bib_records_subjects_links_ibfk_1` FOREIGN KEY (`bib_record_id`) REFERENCES `bib_records` (`id`) ON DELETE CASCADE,
  CONSTRAINT `bib_records_subjects_links_ibfk_2` FOREIGN KEY (`auth_link_id`) REFERENCES `authority_records_links` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 ;
