package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.model.persistence.Filter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Date;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @author Rob
 *
 */
@SuppressWarnings("serial")
public class FilterSettingsPanel extends JPanel {
	
	// Default width (in pixels) of the fields for min/max confidence thresholds
	public static final int CONFIDENCE_FIELD_WIDTH = 50;
	public static final int DATE_FIELD_WIDTH = 150;
	
	public static final int PADDING = 20;

	private Filter filter = new Filter();
	private ResultsTable resultsTable;
	
	public FilterSettingsPanel( ResultsTable resultsTable ) {
		this.resultsTable = resultsTable;
		
		FlowLayout layout = new FlowLayout();
		layout.setAlignment( FlowLayout.LEFT );
		JPanel centerPanel = new JPanel( layout );		
		
		centerPanel.add( createFlaggedCheckbox() );
		centerPanel.add( createDivider() );
		centerPanel.add( createVerificationCheckboxes() );
		centerPanel.add( createDivider() );
		centerPanel.add( createConfidenceRangeFields() );
		centerPanel.add( createDivider() );
		centerPanel.add( createDateRangeFields() );
	
		this.setLayout( new BorderLayout() );
		this.add( new JLabel("Enter limitations about which links should be shown.  The table will automatically refresh."), BorderLayout.NORTH );
		this.add( centerPanel, BorderLayout.SOUTH );
	}
	
	private static JComponent createDivider() {
		// This is so lame, but I don't know of a component that will make a divider like this
		JComponent retVal =  new JComponent() {
			public void paint( Graphics g ) {
				g.setColor( Color.GRAY );
				g.drawLine( 1,1,1,this.getHeight() );
			}
		};
		retVal.setPreferredSize( new Dimension(3, 90) );
		return retVal;
	}

	private JComponent createFlaggedCheckbox() {
		JPanel panel = new JPanel();
		final JCheckBox flaggedCheckBox = new JCheckBox("Show only flagged", false);
		flaggedCheckBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filter.setShowOnlyFlagged( flaggedCheckBox.isSelected() );
				resultsTable.refresh();
			}
		});
		panel.add( flaggedCheckBox );
		return panel;
	}
	
	private JComponent createVerificationCheckboxes() {
		JPanel panel = new JPanel();
		panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
		
		panel.add( new JLabel("Show:") );
		final JCheckBox unverifiedCheckBox = new JCheckBox("unverified", true);
		unverifiedCheckBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filter.setShowUnverified( unverifiedCheckBox.isSelected() );
				resultsTable.refresh();
			}
		});
		panel.add( unverifiedCheckBox );
		final JCheckBox acceptedCheckBox = new JCheckBox("accepted", true);
		acceptedCheckBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filter.setShowAccepted( acceptedCheckBox.isSelected() );
				resultsTable.refresh();
			}
		});
		panel.add( acceptedCheckBox );
		final JCheckBox rejectedCheckBox = new JCheckBox("rejected", true);
		rejectedCheckBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filter.setShowRejected( rejectedCheckBox.isSelected() );
				resultsTable.refresh();
			}
		});
		panel.add( rejectedCheckBox );
		
		return panel;
	}
	
	private JComponent createConfidenceRangeFields() {
		final NumberFormat numberFormat = NumberFormat.getPercentInstance();
		final JFormattedTextField thresholdMinField = new JFormattedTextField();
		final JFormattedTextField thresholdMaxField = new JFormattedTextField();
		
		thresholdMinField.setText("0.0%");
		thresholdMaxField.setText("100%");
		
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				JTextField sourceField = (JTextField)e.getSource();
				if ( !sourceField.getText().trim().endsWith("%") ) {
					sourceField.setText( sourceField.getText().trim() + "%" );
				}
				try {
					filter.setMinConfidence( numberFormat.parse(thresholdMinField.getText()).doubleValue() );
					filter.setMaxConfidence( numberFormat.parse(thresholdMinField.getText()).doubleValue() );
					resultsTable.refresh();
				} catch (ParseException e1) {
					JOptionPane.showMessageDialog( FilterSettingsPanel.this,
							"A problem occured while trying to read the percentage you entered." +
							"\n\nTechnical Reason:\n    " + e1.getMessage());
				}
			}
		};
		thresholdMinField.addActionListener( actionListener );
		thresholdMaxField.addActionListener( actionListener );

		return arrangeMaxMinFields( "Confidence", CONFIDENCE_FIELD_WIDTH, thresholdMinField, thresholdMaxField );
	}
	
	private JComponent createDateRangeFields() {
		final DateFormat dateFormat = ResultsTable.DATE_FORMAT;
		final JFormattedTextField dateMinField = new JFormattedTextField( dateFormat );
		final JFormattedTextField dateMaxField = new JFormattedTextField( dateFormat );
				
		dateMinField.setText( dateFormat.format( new Date() ) );
		dateMaxField.setText( dateFormat.format( new Date() ) );
		
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				try {
					filter.setOldest( dateFormat.parse(dateMinField.getText()) );
					filter.setNewest( dateFormat.parse(dateMaxField.getText()) );
					resultsTable.refresh();
				} catch (ParseException e1) {
					JOptionPane.showMessageDialog( FilterSettingsPanel.this,
							"A problem occured while trying to read the date you entered." +
							"\n\nTechnical Reason:\n    " + e1.getMessage());
				}
			}
		};
		dateMinField.addActionListener( actionListener );
		dateMaxField.addActionListener( actionListener );

		return arrangeMaxMinFields( "Date", DATE_FIELD_WIDTH, dateMinField, dateMaxField );
	}
	
	private static JComponent arrangeMaxMinFields( String label, int width, JTextField minField, JTextField maxField ) {
		// set the width of ihe input fields
		minField.setPreferredSize( new Dimension( width, (int)minField.getPreferredSize().getHeight() ) );
		maxField.setPreferredSize( new Dimension( width, (int)maxField.getPreferredSize().getHeight() ) );
		
		// arrange three panels in a column: one for the label, the next two for the fields
		JPanel panel = new JPanel( new GridLayout( 3, 1 ) );
		JPanel minPanel = new JPanel();
		JPanel maxPanel = new JPanel();
		panel.add( new JLabel( label + " range:" ) );
		panel.add( minPanel );
		panel.add( maxPanel );
		
		// place the field components into the created layout
		minPanel.add( new JLabel("Min:") );
		minPanel.add( minField );
		maxPanel.add( new JLabel("Max:" ) );
		maxPanel.add( maxField );
		
		return panel;
	}
	
	public Filter getFilter() {
		return filter;
	}
	
}
