package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.importer.RecordImporter;
import info.extensiblecatalog.nacm.matcher.MatchDriver;
import info.extensiblecatalog.nacm.model.persistence.AuthorityLinkDAO;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class GUI {
	
	public static final int DEFAULT_WINDOW_WIDTH  = 800;
	public static final int DEFAULT_WINDOW_HEIGHT = 600;
	
	private JFrame mainWindow;
	
	public GUI( RecordImporter importer, MatchDriver matcher, AuthorityLinkDAO database ) {
		
		JobScheduler jobScheduler = new JobScheduler( importer, matcher );
		ResultsTable resultsTable = new ResultsTable( database );
		StatisticsFooter statistics = new StatisticsFooter( database );
		
		JPanel windowContent = arrangeWindowContent( jobScheduler,
													 resultsTable,
													 statistics );
		mainWindow = wrapInWindow( windowContent );
		jobScheduler.hideErrorLog();	// I don't know why, but this has to be called after 
	}
	
	private static JPanel arrangeWindowContent( JComponent jobScheduler, 
												final ResultsTable resultsTable,
												StatisticsFooter statistics ) {
		final JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add( "Schedule Jobs", jobScheduler );
		tabbedPane.add( "Results", resultsTable );
		
		JPanel windowContent = new JPanel( new BorderLayout() );
		windowContent.add( tabbedPane, BorderLayout.CENTER );
		//windowContent.add( statistics, BorderLayout.SOUTH );
		
		// refresh when you change tabs
		tabbedPane.addChangeListener( new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if ( tabbedPane.getSelectedIndex() == 1 ) {
					System.out.println("Table refreshed.");
					resultsTable.refresh();
				}
			}
		});
		
		return windowContent;
	}
	
	private static JFrame wrapInWindow( JPanel windowContent ) {
		JFrame frame = new JFrame("Name Access Control Module - Testing Interface");
		frame.setPreferredSize( new Dimension( DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT ) );
		frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		frame.getContentPane().add( windowContent );
		frame.pack();
		return frame;
	}
	
	public void show() {
		mainWindow.setVisible( true );
	}

}
