package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.gui.scheduling.Job;
import info.extensiblecatalog.nacm.gui.scheduling.TaskPanel;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.util.Date;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

@SuppressWarnings("serial")
public class JobDialog extends JDialog {
	
	public static final String[] RUN_AT_OPTIONS = { "immediately.", "at this date & time:", "after this job completes:" };
	public static final String[] RUN_AT_OPTIONS_LIMITED = { RUN_AT_OPTIONS[0], RUN_AT_OPTIONS[1] };
	
	private JButton okButton;
	private JComboBox runAtMenu;
	private JPanel centerPanel;

	// only used sometimes
	private JTextField dateField;
	private JComboBox prerequsiteJob;
	private Vector<Job> uncompletedJobs;
	
	private JobScheduler jobList;
	private TaskPanel taskPanel;
	
	public JobDialog( TaskPanel taskPanel, JobScheduler jobList ) {
		//super.setUndecorated( true ); // remove close button from window's title bar
		this.setModal( true );
		this.setTitle( taskPanel.getDialogTitle() );
		
		this.jobList = jobList;
		this.taskPanel = taskPanel;

		Container contentPane = getContentPane();
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( taskPanel, BorderLayout.NORTH );
		contentPane.add( createCenterPanel(), BorderLayout.CENTER );
		contentPane.add( createOkCancelButtons(), BorderLayout.SOUTH );
		
		this.setResizable( false );	// such a lame solution.  I'll fix this later
		this.pack();
	}
	
	private JComponent createCenterPanel() {
		centerPanel = new JPanel( new BorderLayout() );
		
		uncompletedJobs = jobList.getUncompletedJobs();
		if ( uncompletedJobs.size() > 0 ) {
			prerequsiteJob = new JComboBox( uncompletedJobs );
			prerequsiteJob.setSelectedIndex( uncompletedJobs.size() - 1 );
		}
		
		dateField = new JFormattedTextField( JobScheduler.DATE_FORMAT );
		dateField.setText( JobScheduler.DATE_FORMAT.format( new Date() ) + "  " );
		
		centerPanel.add( createSchedulingPanel(), BorderLayout.WEST  );
		centerPanel.add( new JPanel() );
		
		return centerPanel;
	}

	private JComponent createSchedulingPanel() {
		JLabel label = new JLabel("This task should run");
		String[] options = RUN_AT_OPTIONS;
		if ( prerequsiteJob == null ) {
			options = RUN_AT_OPTIONS_LIMITED;
		}
		runAtMenu = new JComboBox( options );
		runAtMenu.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				centerPanel.remove(1);
				
				// switch out what goes in centerPanel SOUTH
				JPanel panel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
				if ( runAtMenu.getSelectedIndex() == 0 ) {
					// do nothing, we want an empty panel
					;
				} else if ( runAtMenu.getSelectedIndex() == 1 ) {
					panel.add( dateField );
				} else  {
					panel.add( prerequsiteJob );
				}
				
				centerPanel.add( panel, BorderLayout.CENTER );
				JobDialog.this.pack();
			}
		});
		
		JPanel panel = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
		panel.add( label );
		panel.add( runAtMenu );
		return panel;
	}
	
	private JComponent createOkCancelButtons() {
		okButton = new JButton("Okay");
		JButton cancelButton = new JButton("Cancel");
		
		okButton.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					createJob();
				} catch (ParseException e1) {
					JOptionPane.showMessageDialog( null, "The date you entered could not be understood.");
				}
				JobDialog.this.setVisible( false );
			}
		});
		cancelButton.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// do nothing more than hide the dialog
				JobDialog.this.setVisible( false );
			}
		});
		
		JPanel panel = new JPanel();
		panel.add( okButton );
		panel.add( cancelButton );
		return panel;
	}
	
	private void createJob() throws ParseException {
		Job j = null;
		if ( runAtMenu.getSelectedIndex() == 0 ) {
			j = new Job( jobList, taskPanel.getTask() );
		} else if ( runAtMenu.getSelectedIndex() == 1 ) {
			Date scheduledTime = JobScheduler.DATE_FORMAT.parse( dateField.getText() );
			j = new Job( jobList, taskPanel.getTask(), scheduledTime );
		} else if ( runAtMenu.getSelectedIndex() == 2 ) {
			j = new Job( jobList, taskPanel.getTask(), (Job)prerequsiteJob.getSelectedItem() );
		}
		
		jobList.addJob( j );
	}


}
