/*
 * LinkControls.java
 * Created on Jan 26, 2007 (3:41:47 PM) by Rob Busack
 * 
 * Version:
 * 	$Id: LinkControls.java,v 1.5 2007/01/29 06:53:07 rob.busack Exp $
 * 
 * Revision:
 *  $Log: LinkControls.java,v $
 *  Revision 1.5  2007/01/29 06:53:07  rob.busack
 *  Added an extra JLabel, labeling the verification menu.
 *
 *  Revision 1.4  2007/01/29 06:47:43  rob.busack
 *  Corrected the layout of the link editing controls.
 *
 *  Revision 1.3  2007/01/28 20:34:44  rob.busack
 *  Working on Issue #7
 *
 *  Revision 1.2  2007/01/27 19:06:52  drzonca
 *  changed status booleans to a single status field in AuthorityLinks
 *
 *  Revision 1.1  2007/01/26 22:45:38  rob.busack
 *  Implemented controls to modify links that show up in the results table.  The alignment of the GUI components isn't quite right yet though.
 *
 */
package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.persistence.AuthorityLinkDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @author Rob
 *
 */
@SuppressWarnings("serial")
public class LinkControls extends JPanel {

	private static final int NAME_FIELD_WIDTH = 150;
	
	public static final int UNVERIFIED	= 0;
	public static final int ACCPETED	= 1;
	public static final int REJECTED	= 2;
	public static final String[] VERIFICATION_MENU_OPTIONS = new String[3];
	{
		VERIFICATION_MENU_OPTIONS[ UNVERIFIED ] = "unverified";
		VERIFICATION_MENU_OPTIONS[ ACCPETED ] = "accepted";
		VERIFICATION_MENU_OPTIONS[ REJECTED ] = "rejected";
	}
	
	private AuthorityLinkDAO database;
	private ResultsTable table;
	private List<AuthorityLink> selectedRows;
	
	private JLabel selectionCountLabel;
	private JButton flagButton;
	private JButton unflagButton;
	private JComboBox verificationMenu;
	private ActionListener verificationMenuActionListener;
	private JTextField nameField;
	
	public LinkControls( AuthorityLinkDAO database, ResultsTable table ) {
		super( new BorderLayout() );
//		GridBagConstraints c = new GridBagConstraints();
		
		this.database = database;
		this.table = table;
		
		// instantiation of components
		selectionCountLabel = new JLabel("", JLabel.CENTER );
		flagButton = new JButton("Flag");
		unflagButton = new JButton("Unflag");
		verificationMenu = new JComboBox( VERIFICATION_MENU_OPTIONS );
		nameField = new JTextField("unspecified user");
		nameField.setPreferredSize( new Dimension( NAME_FIELD_WIDTH, nameField.getPreferredSize().height ) );
		
		addActionListeners();
		
		// initialize values for components
		setSelection( new LinkedList<AuthorityLink>() );
		
//		// layout parameters that affect all components
//		c.anchor = GridBagConstraints.WEST;
//			
//		// layout of components
//		c.gridx = 0;
//		c.gridwidth = GridBagConstraints.REMAINDER;
//		this.add( selectionCountLabel, c );
//		
//		c.gridx = 0;
//		c.gridwidth = 1;
//		this.add( flagButton, c );
//		c.gridx = 1;
//		this.add( verificationMenu );
//		c.gridx = 2;
//		this.add( new JLabel("Your name:"), c );
//		c.gridwidth = GridBagConstraints.REMAINDER;
//		c.gridx = 3;
//		this.add( nameField, c );
//		
//		c.gridx = 0;
//		c.gridwidth = 1;
//		this.add( unflagButton, c );
		
		JPanel leftPanel = new JPanel( new GridLayout( 2, 1 ) );
		JPanel rightPanel = new JPanel();
		
		this.add( selectionCountLabel, BorderLayout.NORTH );
		this.add( leftPanel, BorderLayout.WEST );
		this.add( rightPanel, BorderLayout.EAST );
		leftPanel.add( flagButton );
		leftPanel.add( unflagButton );
		rightPanel.add( new JLabel("Mark selection as") );
		rightPanel.add( verificationMenu );
		rightPanel.add( new JLabel("Your name:") );
		rightPanel.add( nameField );
	}

	private void addActionListeners() {
		flagButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setFlagged( true );
			}
		});
		unflagButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setFlagged( false );
			}
		});
		verificationMenuActionListener = new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				 setVerificationStatus( verificationMenu.getSelectedIndex() );
			}
		};
		verificationMenu.addActionListener( verificationMenuActionListener );
		
	}

	public void setSelection( List<AuthorityLink> selectedRows ) {
		this.selectedRows = selectedRows;
		
		int selectionCount = selectedRows.size();
		selectionCountLabel.setText( selectionCount + " links selected for editing." );
		
		boolean atleastOneSelected = (selectionCount != 0);
		boolean atleastOneFlagged = false;
		boolean atleastOneUnflagged = false;
		for ( AuthorityLink link : selectedRows ) {
			atleastOneFlagged = atleastOneFlagged || link.isFlagged();
			atleastOneUnflagged = atleastOneUnflagged || !link.isFlagged();
		}
		
		flagButton.setEnabled( atleastOneUnflagged );
		unflagButton.setEnabled( atleastOneFlagged );
		verificationMenu.setEnabled( atleastOneSelected );
		
		if ( atleastOneSelected ) {
			verificationMenu.removeActionListener( verificationMenuActionListener );
			verificationMenu.setSelectedIndex( selectedRows.get( 0 ).getStatus() );
			verificationMenu.addActionListener( verificationMenuActionListener );
		}
	}
	
	private void setFlagged( boolean flagged ) {
		for ( AuthorityLink link : selectedRows ) {
			link.setFlagged( flagged );
			try {
				database.update( link );
			} catch (DataAccessException e) {
				// TODO show a dialog reporting the failure
				e.printStackTrace();
			}
		}
		table.refresh();
	}
	
	private void setVerificationStatus( int verification ) {
		for ( AuthorityLink link : selectedRows ) {
			switch ( verification ) {
			case UNVERIFIED:
				link.setStatus(AuthorityLink.NOT_REVIEWED);
				break;
			case ACCPETED:
				link.setStatus(AuthorityLink.MARKED_ACCEPTED);
				break;
			case REJECTED:
				link.setStatus(AuthorityLink.MARKED_REJECTED);
				break;
			}
			link.setApprovedBy( nameField.getText() );
			
			try {
				database.update( link );
			} catch (DataAccessException e) {
				// TODO show a dialog reporting the failure
				e.printStackTrace();
			}
		}
		table.refresh();
	}
	
}
