package info.extensiblecatalog.nacm.gui.scheduling;

import info.extensiblecatalog.nacm.importer.RecordImporter;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

public class ImportTask extends Task {
	
	private RecordImporter importer;
	private File fileOrDirectory;
	private boolean recurseNestedDirectories;
	
	private boolean currentlyCountingFiles;
	
	public ImportTask( RecordImporter importer, File fileOrDirectory, boolean recurseNestedDirs ) {
		this.importer = importer;
		this.fileOrDirectory = fileOrDirectory;
		this.recurseNestedDirectories = recurseNestedDirs;
	}

	@Override
	String getTaskName() {
		if ( fileOrDirectory.isDirectory() ) {
			return "Import all records in " + fileOrDirectory.getName() + " directory.";
		} else {
			return "Import the file " + fileOrDirectory.getName();
		}
	}

	@Override
	public void run() {
		currentlyCountingFiles = true;
		List<File> files = makeListOfFilesToImport();
		setLengthOfTask( files.size() );
		currentlyCountingFiles = false;
		
		int counter = 0;
		
		for ( File f : files ) {
			try {
			importer.importRecord( f );
			setProgress( ++counter );
			} catch ( Exception e ) {
				setFailed( "The file \"" + f.getName() + "\"", e );
			}
		}
		
		setCompletedSuccessfully();
	}
	
	private List<File> makeListOfFilesToImport() {
		List<File> list = null;
		
		if ( !fileOrDirectory.isDirectory() ) {
			list = new LinkedList<File>();
			list.add( fileOrDirectory );
		} else if ( recurseNestedDirectories ) {
			list = flattenDirectory( fileOrDirectory );
		} else {
			File[] fileList = fileOrDirectory.listFiles();
			list = new LinkedList<File>();
			for ( File f : fileList ) {
				if ( !f.isDirectory() ) {
					list.add( f );
				}
			}
		}
		
		return list;
	}
	
	private static List<File> flattenDirectory( File directory ) {
		List<File> list = new LinkedList<File>();
		
		File[] filesInDir = directory.listFiles();
		for ( File f : filesInDir ) {
			if ( !f.isDirectory() ) {
				list.add( f );
			} else {
				list.addAll( flattenDirectory( f ) );
			}
		}
		
		return list;
	}
	
	@Override
	public String getProgressString() {
		if ( currentlyCountingFiles ) {
			return "Counting files";
		} else {
			return super.getProgressString();
		}
	}

}
