package info.extensiblecatalog.nacm.gui.scheduling;

import info.extensiblecatalog.nacm.importer.RecordImporter;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

@SuppressWarnings("serial")
public class ImportTaskPanel extends TaskPanel {
	
	private static JFileChooser staticFileChooser = createFileChooser();
	private static String lastPath = File.listRoots()[0].getPath();

	private RecordImporter importer;
	
	private JTextField pathField;
	private JCheckBox recursiveCheckbox;
	
	public ImportTaskPanel( RecordImporter importer ) {
		this.importer = importer;
		
		GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        this.setLayout( gridbag );
        
        c.anchor = GridBagConstraints.WEST;
        c.ipadx = 2;
        c.ipady = 2;
		
        c.fill = GridBagConstraints.NONE;
        c.gridwidth = GridBagConstraints.REMAINDER;
		this.add( new JLabel("Select a file or directory of files:"), c );
		
		c.fill = GridBagConstraints.HORIZONTAL;
        c.gridwidth = GridBagConstraints.RELATIVE;
		this.add( createPathField(), c );
		
		c.fill = GridBagConstraints.NONE;
        c.gridwidth = GridBagConstraints.REMAINDER;
		this.add( createBrowseButton(), c );
		
		// no need to change c
		this.add( createRecursiveCheckbox(), c );
	}

	private JComponent createPathField() {
		//JPanel panel = new JPanel();
		
		pathField = new JTextField();
		pathField.setText( lastPath );
		pathField.setPreferredSize( new Dimension( 600, pathField.getPreferredSize().height ) );
		pathField.selectAll();
			
		//return panel;
		return pathField;
	}
	
	private JComponent createBrowseButton() {
		JButton fileChooserButton = new JButton("Browse...");
		fileChooserButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				File f = getChoosenFile();
				staticFileChooser.setCurrentDirectory( f.isDirectory() ? f.getParentFile() : f );
				int returnVal = staticFileChooser.showDialog( ImportTaskPanel.this, "Import" );
				if ( returnVal == JFileChooser.APPROVE_OPTION ) {
					pathField.setText( staticFileChooser.getSelectedFile().getAbsolutePath() );
				}
			}
		});
		
//		JPanel panel = new JPanel();
//		panel.add( fileChooserButton );
//		return panel;
		return fileChooserButton;
	}
	
	private JComponent createRecursiveCheckbox() {
		recursiveCheckbox = new JCheckBox( "Recurse Nested Directories" );
		return recursiveCheckbox;
	}

	public String getDialogTitle() {
		return "Import Records";
	}
	
	@Override
	public Task getTask() {
		File f = getChoosenFile();
		return new ImportTask( importer, f, recursiveCheckbox.isSelected() );
	}
	
	private File getChoosenFile() {
		lastPath = pathField.getText();
		return new File( lastPath );
	}
	
	private static JFileChooser createFileChooser() {
		JFileChooser fileChooser = new JFileChooser();
		
		FileFilter dirOnlyFiler = new FileFilter() {
			public boolean accept(File f) {
				return f.isDirectory();
			}
			public String getDescription() {
				return "Directories only";
			}
		};
		fileChooser.addChoosableFileFilter( dirOnlyFiler );
		FileFilter xmlFilter = new FileFilter() {
			public boolean accept(File f) {
				return f.getPath().toLowerCase().endsWith(".xml") || f.isDirectory();
			}
			public String getDescription() {
				return "*.xml";
			}
		};
		fileChooser.addChoosableFileFilter( xmlFilter );
		fileChooser.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
		fileChooser.setMultiSelectionEnabled( false ); // users can just select a directory if they want multiple files
		
		return fileChooser;
	}
	
}
