package info.extensiblecatalog.nacm.gui.scheduling;

import info.extensiblecatalog.nacm.matcher.ProgressCallback;
import info.extensiblecatalog.nacm.model.BibliographicRecord;

import javax.swing.JProgressBar;

public abstract class Task
	extends ProgressCallback implements Runnable{
	
	private JProgressBar progressBar = new JProgressBar();
	private Job owner;
	
	private int maxSteps;
	private int completedSteps;
	private int failedSteps;

	/*package*/ abstract String getTaskName();
	
	// I wish I could make this package visability
	public abstract void run();
	
	void setOwnerJob( Job j ) {
		owner = j;
	}
	
	JProgressBar getProgressBar() {
		return progressBar;
	}
	
	// callbacks
	public void setLengthOfTask( int length ) {
		maxSteps = length;
		progressBar.setMaximum( length );
	}
	
	// TODO no longer allow setProgress( int ), only allow incrementCompletedSteps()
	public void setProgress( int progress ) {
		completedSteps = progress;
		progressBar.setValue( progress );
		owner.updateGUI();
	}
	
	
	// TODO it may not be just records that fail, what about files from the importer
	public void itemFailed( BibliographicRecord record, Throwable reason )
	{
		failedSteps++;
		
		// TODO I probably shouldn't be wrapping this liks this.  Solution: Issue #Something
		owner.noteFailure( "Bib Record " + record.getId(), new Exception( reason ) );
	}
	
	// internal use
	void setCompletedSuccessfully() {
		owner.setCompleteSuccessfully();
	}
	
	// TODO  I think this method should be outright removed.
	// Is there any case where an entire Task should be failed, rather
	// than just failing a problematic step and continuing?
	// (This current implementation duplicates the implementation of itemFailed())
	public void setFailed( String source, Exception reason ) {
		failedSteps++;
		owner.noteFailure( source, reason );
	}

	public String getProgressString() {
		// something like "12 of 19 completed, 3 failed"
		return completedSteps + " of " + maxSteps + " succeeded, " + failedSteps + " failed";
	}
	
}
