package info.extensiblecatalog.nacm.importer;

import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.persistence.AuthorityRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.BibliographicRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

/**
 * Determins the type of record being passed to it, then generated the appropriate parser
 * and parses the xml of the passed along record
 * 
 * @author Matthew Horoszowski
 */
public class RecordImporter {
	
	public RecordImporter () {
		
	}
	
	/**
	 * Imports any kind of record into the NACM system
	 * 
	 * @param xmlFile The xml file that is to be importer into the system
	 * @throws FailedImportException 
	 * @throws DataAccessException 
	 */
	public Object importRecord(File xmlFile) throws DataAccessException, FailedImportException {
		
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		Document docRecord = null;
		
		Object returnObject = null;
		
		try {
			//Build out the document object
			DocumentBuilder builder = factory.newDocumentBuilder();
			docRecord = builder.parse( xmlFile );
	 
	    } catch (SAXException sxe) {
	       // Error generated during parsing)
	       Exception  x = sxe;
	       if (sxe.getException() != null) x = sxe.getException();
	       x.printStackTrace();
	       throw new FailedImportException("Invalid XML file", sxe);

	    } catch (ParserConfigurationException pce) {
	    	// Parser with specified options can't be built
	        pce.printStackTrace();

	    } catch (IOException ioe) {
	    	// I/O error
	    	ioe.printStackTrace();
	    }
	    
	    //Determin what kind of record we have
    	
    	System.err.println(xmlFile.getAbsolutePath());
	    
	    if(docRecord.getElementsByTagName("leader").item(0).getTextContent().charAt(6) == 'z') {

	    	//We have an authority file
	    	AuthorityRecordDAO authDAO = new AuthorityRecordDAO();
	    	returnObject = RecordParser.ParseMarcAuthDocument(docRecord);
			authDAO.store((AuthorityRecord) returnObject);
	    	
	    } else {
	    	
	    	//We have an bibliographic file
	    	BibliographicRecordDAO bibDAO = new BibliographicRecordDAO();
	    	returnObject = RecordParser.ParseMarcBibDocument(docRecord);
			bibDAO.store((BibliographicRecord) returnObject);
	    	
	    }
	    
		return returnObject;
	        
	}

}
