package info.extensiblecatalog.nacm.importer;

import java.util.HashSet;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.transformers.StringNormalization;

class RecordParser {
	
	public RecordParser() {
		
	}
	
	/**
	 * Parse a xml dom document and return back a filled out AuthorityRecord
	 * 
	 * @param recordDocument The xml document containig the Marc authority record
	 * @return A authority record holding the information from the Marc xml document
	 * @throws Exception 
	 */
	static AuthorityRecord ParseMarcAuthDocument(Document recordDocument) throws FailedImportException {
		
		AuthorityRecord authRecord = null;
		StringNormalization normalizer = new StringNormalization();
		
		NodeList datafields = recordDocument.getElementsByTagName("datafield");
		
		for(int i = 0; i < datafields.getLength(); i++) {

			if (datafields.item(i).getAttributes().getNamedItem("tag").getFirstChild().getTextContent().equals("100")) {
				
				Name name = new Name(datafields.item(i).getChildNodes().item(1).getTextContent(), 
							         normalizer.transform(datafields.item(i).getChildNodes().item(1).getTextContent())) ;

				authRecord = new AuthorityRecord(name, recordDocument.getTextContent(), 0, false);
				
				break;
				
			}
				
		}
		
		if (authRecord == null) {
			throw new FailedImportException("No authorized name found");
		}
		
		return authRecord;
		
	}

	
	/**
	 * Parse a xml dom document and return back a filled out BibliographicRecord
	 * 
	 * @param recordDocument The xml document containig the Marc bibliographic record
	 * @return A bibliographic record holding the information from the Marc xml document
	 * @throws Exception 
	 */
	static BibliographicRecord ParseMarcBibDocument(Document recordDocument) throws FailedImportException {
		
		BibliographicRecord bibRecord = null;
		StringNormalization normalizer = new StringNormalization();
		
		NodeList datafields = recordDocument.getElementsByTagName("datafield");
		
		HashSet<Name> authors = new HashSet<Name>();
		
		for(int i = 0; i < datafields.getLength(); i++) {

			if (datafields.item(i).getAttributes().getNamedItem("tag").getFirstChild().getTextContent().equals("100")) {
				
				authors.add(new Name(datafields.item(i).getChildNodes().item(1).getTextContent(),
									 normalizer.transform(datafields.item(i).getChildNodes().item(1).getTextContent())));
				
			}
				
		}
		
		if (authors.size() < 1) {
			throw new FailedImportException("No authorized name found");
		}
		
		bibRecord = new BibliographicRecord(null, 0);
		
		bibRecord.setAuthors(authors);
		
		return bibRecord;
		
	}
}
