package info.extensiblecatalog.nacm.matcher;

import info.extensiblecatalog.nacm.model.persistence.*;
import info.extensiblecatalog.nacm.model.*;

import java.util.Set;
import java.util.HashSet;
import java.util.List;

/**
 * A matching strategy that looks for exact name matches in 
 * authority records
 * @author Ben Greenwood
 *
 */
class MatchAuthorizedForm extends MatchStrategy {
	
	AuthorityRecordDAO authDAO;
	
	/**
	 * A constructor to set up the DAO since we only need
	 * to do this once.
	 */
	public MatchAuthorizedForm(){
		authDAO = new AuthorityRecordDAO();
	}
	
	/**
	 * This strategy attempts to find authority records that exactly
	 * match the author name(s) in the bib record.
	 * @param bib The bib record to find matches for
	 * @return a set of authority links created with the information,
	 * the set will be empty if no matches are found 
	 */
	public Set<AuthorityLink> performMatching(Name curr_auth){
		
		Set<AuthorityLink> links = new HashSet<AuthorityLink>();
		
		List<AuthorityRecord> hits = authDAO.findByAuthorizedName(curr_auth.getNormalizedForm());
		if ( hits.size()>0){
			links.addAll( createAuthLinks( hits, curr_auth, 100/hits.size(), true, "Exact match"));
		}
	
		return links;
	}
}