package info.extensiblecatalog.nacm.matcher.test;

import info.extensiblecatalog.nacm.matcher.MatchDriver;
import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.model.persistence.AuthorityLinkDAO;
import info.extensiblecatalog.nacm.model.persistence.AuthorityRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.BibliographicRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

public class MatchDriverTest extends TestCase {
	
	MatchDriver match_driver;

	public void setUp() throws Exception {
		super.setUp();
		match_driver = new MatchDriver();
	}
	
	public void testDriver(){
		Name bob = new Name("Bob McTest","Bob McTest");
		Name bob2 = new Name("Bob McTest","Bob McTest");
		Name mike = new Name("Mike McNoMatch","Mike McNoMatch");
		Name excludee = new Name("Excludee","Excludee");
		AuthorityRecord auth = new AuthorityRecord(bob, "XML!", 2, false);
		
		AuthorityRecord auth2 = new AuthorityRecord(excludee, "not XML!", 5, false);
		
		AuthorityRecordDAO auth_dao = new AuthorityRecordDAO();
		BibliographicRecord bib_record1 = new BibliographicRecord("xml", 3);
		Set<Name> nset = new HashSet<Name>();
		nset.add(bob2);
		bib_record1.setAuthors( nset );
		bib_record1.setMatched(false);
		
		BibliographicRecord bib_record2 = new BibliographicRecord("xml", 3);
		Set<Name> nset2 = new HashSet<Name>();
		nset2.add(mike);
		bib_record2.setAuthors( nset2 );
		bib_record2.setMatched(false);
		
		BibliographicRecordDAO bib_dao = new BibliographicRecordDAO();
		
		AuthorityLinkDAO link_dao = new AuthorityLinkDAO();
		
		try { 
			auth_dao.store(auth);
			auth_dao.store(auth2);
			bib_dao.store(bib_record1);
			bib_dao.store(bib_record2);
			
			int old_count = link_dao.countAllLinks();
			
			match_driver.match();
			
			List<BibliographicRecord> unmatched =  bib_dao.findAllUnmatched();
			
			assertTrue( unmatched.contains(bib_record2) );
			
			assertTrue( link_dao.countAllLinks() > old_count );
			
			List<BibliographicRecord> blist;
			blist = bib_dao.findByAuthorityRecord(auth);
			
			assertTrue( blist.size() >= 1 );
			
			assertTrue(blist.contains(bib_record1));
			
			Set<AuthorityLink> allist;
			allist = bib_record1.getAuthorizedAuthors();
			
			assertTrue( allist.size()>=1);

			bib_dao.remove(bib_record1);
			bib_dao.remove(bib_record2);
			
			auth_dao.remove(auth);
			auth_dao.remove(auth2);
			
		} catch (DataAccessException e){
			assertTrue(false); // an exception is a failure
		}
	}
}