package info.extensiblecatalog.nacm.model;

/**
 * Represents a single name in the model
 * @author Dean Rzonca
 *
 */
public class Name
{
	Long id;
	String originalForm;
	String normalizedForm;
	
	/**
	 * Creates a new, empty Name
	 * This constructor is only provided for JavaBeans
	 * compliance
	 */
	public Name()
	{
		id = -1l;
		originalForm = "not specified";
		normalizedForm = "not specified";
	}
	
	/**
	 * Creates a new Name
	 * @param original		the name, exactly as entered
	 * @param normalized	the normalized form of the name
	 */
	public Name(String original, String normalized)
	{
		id = -1l;
		originalForm = original;
		normalizedForm = normalized;
	}

	
	/**
	 * Gets the unique ID associated with this record
	 * @return	a unique ID
	 */
	public Long getId()
	{
		return id;
	}

	/**
	 * Sets the unique ID associated with this record
	 * @param id	a unique ID
	 */
	public void setId(Long id)
	{
		this.id = id;
	}

	/**
	 * Gets the normalized form of this name
	 * @return	the name
	 */
	public String getNormalizedForm()
	{
		return normalizedForm;
	}

	/**
	 * Sets the normalized form of this name
	 * @param normalizedForm	the name
	 */
	public void setNormalizedForm(String normalizedForm)
	{
		this.normalizedForm = normalizedForm;
	}

	/**
	 * Gets the original form of this name, exactly as entered
	 * @return	the name
	 */
	public String getOriginalForm()
	{
		return originalForm;
	}

	/**
	 * Sets the original form of this name, exactly as entered
	 * @param originalForm	the name
	 */
	public void setOriginalForm(String originalForm)
	{
		this.originalForm = originalForm;
	}


	@Override
	public boolean equals(Object obj)
	{
		boolean equal = false;
		
		if(obj instanceof Name)
		{
			Name other = (Name) obj;
			
			equal = 
				id.equals(other.getId()) &&
				originalForm.equals(other.getOriginalForm()) &&
				normalizedForm.equals(other.getNormalizedForm());
		}
		
		return equal;
	}


	@Override
	public int hashCode()
	{
		return
			originalForm.hashCode() +
			normalizedForm.hashCode();
	}


	@Override
	public String toString()
	{
		return originalForm;
	}
	
	
}