package info.extensiblecatalog.nacm.model.persistence;

import info.extensiblecatalog.nacm.model.AuthorityLink;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Provides search functionality for generating
 * reports with AuthorityLinks
 * @author Dean Rzonca
 *
 */
public class AuthorityLinkDAO
{
	static final String LINK_CLASS = AuthorityLink.class.getName();
	
	HibernateHelper helper = new HibernateHelper();
	
	/**
	 * Updates an authority link
	 * An authority link cannot exist without a bibliographic record,
	 * so only updating existing records is permitted through this interface
	 * @param link	the link to update
	 * 
	 * @throws DataAccessException	if the link does not already exist
	 */
	public void update(AuthorityLink link)
	throws DataAccessException
	{
		if(!link.getId().equals(-1l))
		{
			helper.store(link);
		}
		else
		{
			throw new DataAccessException("Cannot create AuthorityLink without a bib record");
		}
	}
	
	/**
	 * Finds all authority links and applies a filter
	 * @param filter	the filter
	 * @return			a List of AuthorityLinks that matches the filter
	 */
	public List<AuthorityLink> findAllWithFilter(Filter filter)
	{
		return null;
	}
	
	/**
	 * Finds all authority links
	 * @return			a List of AuthorityLinks that matches the filter
	 */
	public List<AuthorityLink> findAll()
	{
		return castList(
				helper.getSession().createQuery("from " + LINK_CLASS).list());
	}
	
	/**
	 * Counts all links between bib records and authority records
	 * presently in the system
	 * @return	
	 */
	public int countAllLinks()
	{
		return helper.getSession().createQuery("from " + LINK_CLASS).list().size();
	}
	
	protected List<AuthorityLink> castList(List unsafe)
	{
		List<AuthorityLink> safe = new LinkedList<AuthorityLink>();
		Iterator unsafeItems = unsafe.iterator();
		
		while(unsafeItems.hasNext())
		{
			safe.add((AuthorityLink) unsafeItems.next());
		}
		
		return safe;
	}
}