package info.extensiblecatalog.nacm.model.persistence.test;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.model.persistence.AuthorityLinkDAO;
import info.extensiblecatalog.nacm.model.persistence.AuthorityRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.BibliographicRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.util.List;

import junit.framework.TestCase;

public class AuthorityLinkDAOTest extends TestCase
{
	AuthorityLinkDAO dao;

	public void setUp() throws Exception
	{
		super.setUp();
		
		dao = new AuthorityLinkDAO();
	}

	public void testUpdate()
	{
		// Create a new AuthorityLink, without a bib record
		AuthorityRecord auth = new AuthorityRecord();
		AuthorityLink link = new AuthorityLink();
		link.setRecord(auth);
		link.setRelatedName(new Name());
		
		// Attempt to store it, this should fail
		try
		{
			dao.update(link);
			fail("Creating standalone link should not be permitted");
		}
		catch(DataAccessException e)
		{
			// Nothing to do here
		}
	}

	public void testCountAllLinks()
	throws DataAccessException
	{
		// Create a valid link
		BibliographicRecord bibRecord = new BibliographicRecord();
		AuthorityRecord authRecord = new AuthorityRecord();
		Name author = new Name();
		// Note that bib records and auth records should not share name references
		Name authorizedAuthor = new Name();
		bibRecord.getAuthors().add(author);
		authRecord.setAuthorizedForm(authorizedAuthor);
		
		AuthorityLink link = new AuthorityLink();
		link.setRecord(authRecord);
		link.setRelatedName(author);
		bibRecord.getAuthorizedAuthors().add(link);
		
		// Find out how many links exist at the moment
		int count = dao.countAllLinks();
		
		// Store the link
		BibliographicRecordDAO bibDAO = new BibliographicRecordDAO();
		bibDAO.store(bibRecord);
		
		// Find out how many there are now
		int newCount = dao.countAllLinks();
		
		// Make sure the number went up
		assertEquals("Count did not increase",count+1,newCount);
		
		// Now, remove the link
		bibRecord.getAuthorizedAuthors().clear();
		
		// Store it
		bibDAO.store(bibRecord);
		
		// Make sure the count went back down
		newCount = dao.countAllLinks();
		
		// Clean up
		bibDAO.remove(bibRecord);
		new AuthorityRecordDAO().remove(authRecord);
	}
	
	public void testFindAll()
	throws DataAccessException
	{
		// Create a valid link
		BibliographicRecord bibRecord = new BibliographicRecord();
		AuthorityRecord authRecord = new AuthorityRecord();
		Name author = new Name();
		// Note that bib records and auth records should not share name references
		Name authorizedAuthor = new Name();
		bibRecord.getAuthors().add(author);
		authRecord.setAuthorizedForm(authorizedAuthor);
		
		AuthorityLink link = new AuthorityLink();
		link.setRecord(authRecord);
		link.setRelatedName(author);
		bibRecord.getAuthorizedAuthors().add(link);
		
		// Find out how many links exist at the moment
		int count = dao.findAll().size();
		
		// Store the link
		BibliographicRecordDAO bibDAO = new BibliographicRecordDAO();
		bibDAO.store(bibRecord);
		
		// Find out how many there are now
		List<AuthorityLink> found = dao.findAll();
		int newCount = found.size();
		
		// Make sure the number went up
		assertEquals("Count did not increase",count+1,newCount);
		
		// Make sure the link is in the results
		assertTrue("Link was not in results",found.contains(link));
		
		// Now, remove the link
		bibRecord.getAuthorizedAuthors().clear();
		
		// Store it
		bibDAO.store(bibRecord);
		
		// Make sure the count went back down
		newCount = dao.findAll().size();
		
		// Clean up
		bibDAO.remove(bibRecord);
		new AuthorityRecordDAO().remove(authRecord);
	}

}