package info.extensiblecatalog.nacm.transformers;

/**
 * A single technique that can be used to manipulate
 * a string
 * 
 * @author Dean Rzonca
 *
 */
public abstract class StringTransformer
{	
	static final int NO_CONFIDENCE = 0;
	
	/**
	 * Performs the manipulation on the given string
	 * @param source	the string to transform
	 * @return			the manipulated form of the string
	 */
	abstract public String transform(String source);
	
	/**
	 * Gets the likelihood that another string
	 * that is identical to a manipulated form
	 * produced by this StringTransformer represents
	 * the same thing as the original string used
	 * when performing the manipulation
	 * @return	a whole number percentage, from 0-100
	 */
	public int getConfidenceFactor()
	{
		return NO_CONFIDENCE;
	}
}